
import glslang from "./libs/glslang";
import { wtContext } from "wtek/wtContext.js";
import { wtApp } from "./wtApp.js";

// globals /////////////////////////////////////////////////////////
var app_ = null;
var canvas = null;
var lastTimeStamp = 0;
////////////////////////////////////////////////////////////////////

let loop = function (timestamp) {

    if (app_ === null)
    {
        alert("app null");
        return;
    }

    let deltaTime = timestamp - lastTimeStamp;

    app_.update(deltaTime / 100);
    app_.render(timestamp);

    lastTimeStamp = timestamp;
    requestAnimationFrame(loop);
}

function getBrowser(sUsrAg)
{
    let sBrowser;
    if (sUsrAg.indexOf("Firefox") > -1) {
        sBrowser = "Firefox";
        // "Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:61.0) Gecko/20100101 Firefox/61.0"
      } else if (sUsrAg.indexOf("SamsungBrowser") > -1) {
        sBrowser = "SamsungInternet";
        // "Mozilla/5.0 (Linux; Android 9; SAMSUNG SM-G955F Build/PPR1.180610.011) AppleWebKit/537.36 (KHTML, like Gecko) SamsungBrowser/9.4 Chrome/67.0.3396.87 Mobile Safari/537.36
      } else if (sUsrAg.indexOf("Opera") > -1 || sUsrAg.indexOf("OPR") > -1) {
        sBrowser = "Opera";
        // "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_14_0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/70.0.3538.102 Safari/537.36 OPR/57.0.3098.106"
      } else if (sUsrAg.indexOf("Trident") > -1) {
        sBrowser = "IExplorer";
        // "Mozilla/5.0 (Windows NT 10.0; WOW64; Trident/7.0; .NET4.0C; .NET4.0E; Zoom 3.6.0; wbx 1.0.0; rv:11.0) like Gecko"
      } else if (sUsrAg.indexOf("Edge") > -1) {
        sBrowser = "Edge";
        // "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.36 Edge/16.16299"
      } else if (sUsrAg.indexOf("Chrome") > -1) {
        sBrowser = "Chrome";
        // "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Ubuntu Chromium/66.0.3359.181 Chrome/66.0.3359.181 Safari/537.36"
      } else if (sUsrAg.indexOf("Safari") > -1) {
        sBrowser = "Safari";
        // "Mozilla/5.0 (iPhone; CPU iPhone OS 11_4 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Version/11.0 Mobile/15E148 Safari/604.1 980x1306"
      } else {
        sBrowser = "unknown";
      }

      return sBrowser;
}

async function init(glslang) {

    const gpu = navigator['gpu'];
    const adapter = await gpu.requestAdapter();
    const device = await adapter.requestDevice();
    var gpuContext = new wtContext(gpu, adapter, device);

    if (!gpuContext.isValid()) {
        console.log('invalid');
        return;
    }

    canvas = document.createElement('canvas');
    canvas.width = gpuContext.getWidth();
    canvas.height = gpuContext.getHeight();
    document.body.appendChild(canvas);
    app_ = new wtApp(gpuContext, glslang, canvas);
    let browserID = getBrowser(navigator.userAgent);
    let frame = await app_.init(browserID);

    // event handling
    window.addEventListener("keydown", function (event) { app_.getKeyboardEventHandler().keyDown(event.key) });
    window.addEventListener("keyup", function (event) { app_.getKeyboardEventHandler().keyUp(event.key) });

    window.addEventListener("mousedown", function (event) { app_.getMouseEventHandler().onMouseDown(event) });
    window.addEventListener("mouseup", function (event) { app_.getMouseEventHandler().onMouseUp(event) });
    window.addEventListener("mousemove", function (event) { app_.getMouseEventHandler().onMouseMove(event) });
    window.addEventListener("mouseleave", function (event) { app_.getMouseEventHandler().onMouseLeave(event) });
    window.addEventListener("click", function (event) { app_.getMouseEventHandler().onMouseClick(event) });
    window.addEventListener("dblclick", function (event) { app_.getMouseEventHandler().onMouseDoubleClick(event) });

    requestAnimationFrame(loop);
}

const ready = glslang();
ready.then(init);
