
class ModelLoader {
    constructor() {
        this.modelLoaded = false;
        this.vertexData_ = [];
    }

    getVertexData() {
        return this.objVertexData_[0];
    }

    addVertex(vert) {
        const ptn = vert.split('/');
        ptn.forEach((objIndexStr, i) => {
            if (!objIndexStr) {
                return;
            }
            const objIndex = parseInt(objIndexStr);
            const index = objIndex + (objIndex >= 0 ? 0 : this.objVertexData_[i].length);
            this.webglVertexData_[i].push(...this.objVertexData_[i][index]);
        });
    }

    parseObjFile(text) {
        // infaumous index 1 obj
        let objPositions_ = [[0, 0, 0]];
        let objTexcoords_ = [[0, 0]];
        let objNormals_ = [[0, 0, 0]];

        this.objVertexData_ = [objPositions_, objTexcoords_, objNormals_,];
        this.webglVertexData_ = [
            [],   // positions
            [],   // texcoords
            [],   // normals
        ];

        this.keywords = {
            v(parts) {
                objPositions_.push(parts.map(parseFloat));
            },
            vn(parts) {
                objNormals_.push(parts.map(parseFloat));
            },
            vt(parts) {
                objTexcoords_.push(parts.map(parseFloat));
            },
            f(parts) {

            },
        }

        const keywordRE = /(\w*)(?: )*(.*)/;
        const lines = text.split('\n');
        for (let lineNo = 0; lineNo < lines.length; ++lineNo) {
            const line = lines[lineNo].trim();
            if (line === '' || line.startsWith('#')) {
                continue;
            }
            const m = keywordRE.exec(line);

            if (!m) {
                continue;
            }
            const [, keyword, unparsedArgs] = m;
            const parts = line.split(/\s+/).slice(1);
            const handler = this.keywords[keyword];
            if (!handler) {
                console.warn('unhandled keyword:', keyword);  // eslint-disable-line no-console
                continue;
            }
            handler(parts, unparsedArgs);
        }
    }
}

export { ModelLoader };