
export const MouseButtons =
{
    LeftButton: 0,
    MiddleButton: 1,
    RightButton: 2
}

class MouseEventHandler {

    constructor() {
        this.width_ = 1;
        this.height_ = 1;

        this.mouseClicked_ = [false, false, false];
        this.mouseDoubleClicked_ = [false, false, false];
        this.mouseDown_ = [false, false, false];

        this.screenX_ = 0;
        this.screenY_ = 0;
        this.clientX_ = 0;
        this.clientY_ = 0;

        this.lastX_ = 0;
        this.lastY_ = 0;

        this.deltaMouseX_ = 0.0;
        this.deltaMouseY_ = 0.0;

        this.mouseInside_ = false;
    }

    getMouseDeltaX() { return this.deltaMouseX_; }
    getMouseDeltaY() { return this.deltaMouseY_; }

    isDown(mouseButton) {
        return this.mouseDown_[mouseButton];
    }

    resetMouseState() {
        this.flush();
        this.screenX_ = 0;
        this.screenY_ = 0;
        this.clientX_ = 0;
        this.clientY_ = 0;
        this.lastX_ = 0;
        this.lastY_ = 0;
        this.deltaMouseX_ = 0.0;
        this.deltaMouseY_ = 0.0;

        for (let i = 0; i < 3; i++) {
            this.mouseDown_[i] = false;
        }
    }

    flush() {
        for (let i = 0; i < 3; i++) {
            this.mouseClicked_[i] = false;
            this.mouseDoubleClicked_[i] = false;
        }

        this.deltaMouseX_ = 0.0;
        this.deltaMouseY_ = 0.0;
    }

    onMouseMove(event) {
        this.screenX_ = event.screenX;
        this.screenY_ = event.screenY;
        this.clientX_ = event.clientX;
        this.clientY_ = event.clientY;

        let rect = event.target.getBoundingClientRect();
        this.width_ = 800;// rect.right - rect.left;
        this.height_ = 600; rect.bottom - rect.top;

        if (this.mouseInside_) {
            this.deltaMouseX_ = (this.clientX_ - this.lastX_) / this.width_;
            this.deltaMouseY_ = (this.clientY_ - this.lastY_) / this.height_;
        }

        // Check if the mouse cursor is in canvas. 
        if (this.clientX_ >= rect.left &&
            this.clientX_ < 800 && //rect.right &&
            this.clientY_ >= rect.top &&
            this.clientY_ < rect.bottom) {
            this.lastX_ = this.clientX_;
            this.lastY_ = this.clientY_;
            this.mouseInside_ = true;
        }
        else {
            this.mouseInside_ = false;
        }
    }

    onMouseDown(event) {
        this.mouseDown_[event.button] = true && this.mouseInside_;
    }

    onMouseUp(event) {
        this.mouseDown_[event.button] = false;
    }

    onMouseClick(event) {
        console.log('click');
        this.mouseClicked_[event.button] = true;
    }

    onMouseDoubleClick(event) {
        console.log('double click');
        this.mouseDoubleClicked_[event.button] = true;
    }

    onMouseContextMenu(event) {

    }

    onMouseLeave(event) {
        this.mouseInside_ = false;
        console.log('leave');
        this.resetMouseState();
    }
}

export { MouseEventHandler };